@@ -35,15 +35,17 @@
 
 #include "googlesearch.h"
 
+/* Compatibility */
+#if ! GLIB_CHECK_VERSION(2,22,0)
+#define g_mapped_file_unref g_mapped_file_free
+#endif
+
 #define GOOGLE_GOTO_URL_FMT "http://maps.google.com/maps?q=%s&output=js"
 #define GOOGLE_GOTO_PATTERN_1 "{center:{lat:"
 #define GOOGLE_GOTO_PATTERN_2 ",lng:"
 #define GOOGLE_GOTO_NOT_FOUND "not understand the location"
 
-static DownloadMapOptions googlesearch_options = { FALSE, FALSE, "http://maps.google.com/", 0, a_check_map_file };
-
-static void google_goto_tool_class_init ( GoogleGotoToolClass *klass );
-static void google_goto_tool_init ( GoogleGotoTool *vwd );
+static DownloadMapOptions googlesearch_options = { FALSE, FALSE, "http://maps.google.com/", 2, a_check_map_file };
 
 static void google_goto_tool_finalize ( GObject *gob );
 
@@ -51,29 +53,7 @@ static gchar *google_goto_tool_get_url_format ( VikGotoTool *self );
 static DownloadMapOptions *google_goto_tool_get_download_options ( VikGotoTool *self );
 static gboolean google_goto_tool_parse_file_for_latlon(VikGotoTool *self, gchar *filename, struct LatLon *ll);
 
-GType google_goto_tool_get_type()
-{
-  static GType w_type = 0;
-
-  if (!w_type)
-  {
-    static const GTypeInfo w_info = 
-    {
-      sizeof (GoogleGotoToolClass),
-      NULL, /* base_init */
-      NULL, /* base_finalize */
-      (GClassInitFunc) google_goto_tool_class_init,
-      NULL, /* class_finalize */
-      NULL, /* class_data */
-      sizeof (GoogleGotoTool),
-      0,
-      (GInstanceInitFunc) google_goto_tool_init,
-    };
-    w_type = g_type_register_static ( VIK_GOTO_TOOL_TYPE, "GoogleGotoTool", &w_info, 0 );
-  }
-
-  return w_type;
-}
+G_DEFINE_TYPE (GoogleGotoTool, google_goto_tool, VIK_GOTO_TOOL_TYPE)
 
 static void google_goto_tool_class_init ( GoogleGotoToolClass *klass )
 {
@@ -118,7 +98,7 @@ static gboolean google_goto_tool_parse_file_for_latlon(VikGotoTool *self, gchar
 
   if ((mf = g_mapped_file_new(file_name, FALSE, NULL)) == NULL) {
     g_critical(_("couldn't map temp file"));
-    exit(1);
+    return FALSE;
   }
   len = g_mapped_file_get_length(mf);
   text = g_mapped_file_get_contents(mf);
@@ -168,7 +148,7 @@ static gboolean google_goto_tool_parse_file_for_latlon(VikGotoTool *self, gchar
   ll->lon = g_ascii_strtod(lon_buf, NULL);
 
 done:
-  g_mapped_file_free(mf);
+  g_mapped_file_unref(mf);
   return (found);
 
 }
