@@ -24,7 +24,11 @@
 
 #include "viktrwlayer.h"
 
+G_BEGIN_DECLS
+
 gboolean a_gpspoint_read_file ( VikTrwLayer *trw, FILE *f );
 void a_gpspoint_write_file ( VikTrwLayer *trw, FILE *f );
 
+G_END_DECLS
+
 #endif
