@@ -42,7 +42,7 @@ static List *queue_tail = NULL;
 static int queue_count = 0;
 
 static guint32 queue_size = 0;
-static guint32 max_queue_size = VIK_CONFIG_MAPCACHE_SIZE;
+static guint32 max_queue_size = VIK_CONFIG_MAPCACHE_SIZE * 1024 * 1024;
 
 
 static GHashTable *cache = NULL;
@@ -54,17 +54,17 @@ static GMutex *mc_mutex = NULL;
 
 static VikLayerParamScale params_scales[] = {
   /* min, max, step, digits (decimal places) */
- { 1, 300, 1, 0 },
+ { 1, 1024, 1, 0 },
 };
 
 static VikLayerParam prefs[] = {
-  { VIKING_PREFERENCES_NAMESPACE "mapcache_size", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Mapcache memory size (MB):"), VIK_LAYER_WIDGET_HSCALE, params_scales, NULL },
+  { VIKING_PREFERENCES_NAMESPACE "mapcache_size", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Map cache memory size (MB):"), VIK_LAYER_WIDGET_HSCALE, params_scales, NULL, NULL },
 };
 
 void a_mapcache_init ()
 {
   VikLayerParamData tmp;
-  tmp.u = VIK_CONFIG_MAPCACHE_SIZE / 1024 / 1024;
+  tmp.u = VIK_CONFIG_MAPCACHE_SIZE;
   a_preferences_register(prefs, tmp, VIKING_PREFERENCES_GROUP_KEY);
 
   mc_mutex = g_mutex_new();
@@ -124,7 +124,7 @@ static void list_add_entry ( gchar *key )
   }
 }
 
-void a_mapcache_add ( GdkPixbuf *pixbuf, gint x, gint y, gint z, guint8 type, guint zoom, guint8 alpha, gdouble xshrinkfactor, gdouble yshrinkfactor )
+void a_mapcache_add ( GdkPixbuf *pixbuf, gint x, gint y, gint z, guint16 type, guint zoom, guint8 alpha, gdouble xshrinkfactor, gdouble yshrinkfactor )
 {
   gchar *key = g_strdup_printf ( HASHKEY_FORMAT_STRING, x, y, z, type, zoom, alpha, xshrinkfactor, yshrinkfactor );
   static int tmp = 0;
@@ -155,14 +155,14 @@ void a_mapcache_add ( GdkPixbuf *pixbuf, gint x, gint y, gint z, guint8 type, gu
   if ( (++tmp == 100 ))  { g_print("DEBUG: queue count=%d size=%u\n", queue_count, queue_size ); tmp=0; }
 }
 
-GdkPixbuf *a_mapcache_get ( gint x, gint y, gint z, guint8 type, guint zoom, guint8 alpha, gdouble xshrinkfactor, gdouble yshrinkfactor )
+GdkPixbuf *a_mapcache_get ( gint x, gint y, gint z, guint16 type, guint zoom, guint8 alpha, gdouble xshrinkfactor, gdouble yshrinkfactor )
 {
   static char key[48];
   g_snprintf ( key, sizeof(key), HASHKEY_FORMAT_STRING, x, y, z, type, zoom, alpha, xshrinkfactor, yshrinkfactor );
   return g_hash_table_lookup ( cache, key );
 }
 
-void a_mapcache_remove_all_shrinkfactors ( gint x, gint y, gint z, guint8 type, guint zoom )
+void a_mapcache_remove_all_shrinkfactors ( gint x, gint y, gint z, guint16 type, guint zoom )
 {
   char key[40];
   List *loop = queue_tail;
