@@ -22,11 +22,19 @@
 #ifndef __VIKING_OSM_TRACES_H
 #define __VIKING_OSM_TRACES_H
 
-
 #include <glib.h>
+#include <gtk/gtk.h>
+
+G_BEGIN_DECLS
 
 void osm_traces_init();
 void osm_traces_upload_cb(gpointer layer_and_vlp[2], guint file_type);
-void osm_traces_upload_track_cb(gpointer pass_along[6]);
+void osm_traces_upload_track_cb(gpointer pass_along[8]);
+
+void osm_set_login (const gchar *user_, const gchar *password_);
+gchar *osm_get_login();
+void osm_login_widgets (GtkWidget *user_entry, GtkWidget *password_entry);
+
+G_BEGIN_DECLS
 
 #endif
