@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <glib/gstdio.h>
 #include "preferences.h"
+#include "dir.h"
 #include "file.h"
 
 // TODO: register_group
@@ -62,7 +63,7 @@ static void preferences_groups_uninit()
 void a_preferences_register_group ( const gchar *key, const gchar *name )
 {
   if ( g_hash_table_lookup ( groups_keys_to_indices, key ) )
-    g_error("Duplicate preferences group keys");
+    g_critical("Duplicate preferences group keys");
   else {
     g_ptr_array_add ( groups_names, g_strdup(name) );
     g_hash_table_insert ( groups_keys_to_indices, g_strdup(key), GINT_TO_POINTER ( (gint) groups_names->len ) ); /* index + 1 */
@@ -85,16 +86,15 @@ static gint16 preferences_groups_key_to_index( const gchar *key )
 typedef struct {
   VikLayerParamData data;
   guint8 type;
-  gpointer freeme; // because data.s is const and the compiler complains
 } VikLayerTypedParamData;
 
-void layer_typed_param_data_free(gpointer p)
+static void layer_typed_param_data_free(gpointer p)
 {
   VikLayerTypedParamData *val = (VikLayerTypedParamData *)p;
   switch ( val->type ) {
     case VIK_LAYER_PARAM_STRING:
-      if ( val->freeme )
-        g_free ( val->freeme );
+      if ( val->data.s )
+        g_free ( (gpointer)val->data.s );
       break;
     /* TODO: APPLICABLE TO US? NOTE: string layer works auniquely: data.sl should NOT be free'd when
      * the internals call get_param -- i.e. it should be managed w/in the layer.
@@ -102,13 +102,13 @@ void layer_typed_param_data_free(gpointer p)
      * by the layer -- i.e. free'd by the layer.
      */
     case VIK_LAYER_PARAM_STRING_LIST:
-      g_error ( "Param strings not implemented in preferences"); //fake it
+      g_critical ( "Param strings not implemented in preferences"); //fake it
       break;
   }
   g_free ( val );
 }
 
-VikLayerTypedParamData *layer_typed_param_data_copy_from_data(guint8 type, VikLayerParamData val) {
+static VikLayerTypedParamData *layer_typed_param_data_copy_from_data(guint8 type, VikLayerParamData val) {
   VikLayerTypedParamData *newval = g_new(VikLayerTypedParamData,1);
   newval->data = val;
   newval->type = type;
@@ -116,7 +116,6 @@ VikLayerTypedParamData *layer_typed_param_data_copy_from_data(guint8 type, VikLa
     case VIK_LAYER_PARAM_STRING: {
       gchar *s = g_strdup(newval->data.s);
       newval->data.s = s;
-      newval->freeme = s;
       break;
     }
     /* TODO: APPLICABLE TO US? NOTE: string layer works auniquely: data.sl should NOT be free'd when
@@ -125,14 +124,14 @@ VikLayerTypedParamData *layer_typed_param_data_copy_from_data(guint8 type, VikLa
      * by the layer -- i.e. free'd by the layer.
      */
     case VIK_LAYER_PARAM_STRING_LIST:
-      g_error ( "Param strings not implemented in preferences"); //fake it
+      g_critical ( "Param strings not implemented in preferences"); //fake it
       break;
   }
   return newval;
 }
 
 /* TODO: share this code with file.c */
-VikLayerTypedParamData *layer_data_typed_param_copy_from_string ( guint8 type, const gchar *str )
+static VikLayerTypedParamData *layer_data_typed_param_copy_from_string ( guint8 type, const gchar *str )
 {
   g_assert ( type != VIK_LAYER_PARAM_STRING_LIST );
   VikLayerTypedParamData *rv = g_new(VikLayerTypedParamData,1);
@@ -149,7 +148,6 @@ VikLayerTypedParamData *layer_data_typed_param_copy_from_string ( guint8 type, c
     default: {
       gchar *s = g_strdup(str);
       rv->data.s = s;
-      rv->freeme = s;
     }
   }
   return rv;
@@ -202,7 +200,7 @@ static gboolean preferences_load_from_file()
         // otherwise change it (you know the type!)
         // if it's a string list do some funky stuff ... yuck... not yet.
         if ( oldval->type == VIK_LAYER_PARAM_STRING_LIST )
-          g_error ( "Param strings not implemented in preferences"); // fake it
+          g_critical ( "Param strings not implemented in preferences"); // fake it
 
         newval = layer_data_typed_param_copy_from_string ( oldval->type, val );
         g_hash_table_insert ( values, key, newval );
@@ -222,7 +220,7 @@ static gboolean preferences_load_from_file()
 static void preferences_run_setparam ( gpointer notused, guint16 i, VikLayerParamData data, VikLayerParam *params )
 {
   if ( params[i].type == VIK_LAYER_PARAM_STRING_LIST )
-    g_error ( "Param strings not implemented in preferences"); //fake it
+    g_critical ( "Param strings not implemented in preferences"); //fake it
   g_hash_table_insert ( values, (gchar *)(params[i].name), layer_typed_param_data_copy_from_data(params[i].type, data) );
 }
 
@@ -237,7 +235,7 @@ static VikLayerParamData preferences_run_getparam ( gpointer notused, guint16 i,
   VikLayerTypedParamData *val = (VikLayerTypedParamData *) g_hash_table_lookup ( values, ((VikLayerParam *)g_ptr_array_index(params,i))->name );
   g_assert ( val != NULL );
   if ( val->type == VIK_LAYER_PARAM_STRING_LIST )
-    g_error ( "Param strings not implemented in preferences"); //fake it
+    g_critical ( "Param strings not implemented in preferences"); //fake it
   return val->data;
 }
 
