@@ -29,6 +29,7 @@
 
 static gboolean _coord_to_mapcoord ( VikMapSource *self, const VikCoord *src, gdouble xzoom, gdouble yzoom, MapCoord *dest );
 static void _mapcoord_to_center_coord ( VikMapSource *self, MapCoord *src, VikCoord *dest );
+static gboolean _is_direct_file_access ( VikMapSource *self );
 
 static gchar *_get_uri( VikMapSourceDefault *self, MapCoord *src );
 static gchar *_get_hostname( VikMapSourceDefault *self );
@@ -132,6 +133,7 @@ terraserver_map_source_class_init (TerraserverMapSourceClass *klass)
 	/* Overiding methods */
 	grandparent_class->coord_to_mapcoord =        _coord_to_mapcoord;
 	grandparent_class->mapcoord_to_center_coord = _mapcoord_to_center_coord;
+	grandparent_class->is_direct_file_access = _is_direct_file_access;
 	
 	parent_class->get_uri = _get_uri;
 	parent_class->get_hostname = _get_hostname;
@@ -206,6 +208,12 @@ _coord_to_mapcoord ( VikMapSource *self, const VikCoord *src, gdouble xmpp, gdou
 	return TRUE;
 }
 
+static gboolean
+_is_direct_file_access ( VikMapSource *self )
+{
+	return FALSE;
+}
+
 static void
 _mapcoord_to_center_coord ( VikMapSource *self, MapCoord *src, VikCoord *dest )
 {
@@ -246,7 +254,7 @@ _get_download_options( VikMapSourceDefault *self )
 }
 
 TerraserverMapSource *
-terraserver_map_source_new_with_id (guint8 id, const char *label, int type)
+terraserver_map_source_new_with_id (guint16 id, const char *label, int type)
 {
 	char *copyright = NULL;
 	switch (type)
