@@ -115,12 +115,12 @@ enum { DEM_TYPE_HEIGHT = 0,
 };
 
 static VikLayerParam dem_layer_params[] = {
-  { "files", VIK_LAYER_PARAM_STRING_LIST, VIK_LAYER_GROUP_NONE, N_("DEM Files:"), VIK_LAYER_WIDGET_FILELIST },
+  { "files", VIK_LAYER_PARAM_STRING_LIST, VIK_LAYER_GROUP_NONE, N_("DEM Files:"), VIK_LAYER_WIDGET_FILELIST, NULL, NULL, NULL },
   { "source", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Download Source:"), VIK_LAYER_WIDGET_RADIOGROUP_STATIC, params_source, NULL },
-  { "color", VIK_LAYER_PARAM_COLOR, VIK_LAYER_GROUP_NONE, N_("Min Elev Color:"), VIK_LAYER_WIDGET_COLOR, 0 },
-  { "type", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Type:"), VIK_LAYER_WIDGET_RADIOGROUP_STATIC, params_type, NULL },
-  { "min_elev", VIK_LAYER_PARAM_DOUBLE, VIK_LAYER_GROUP_NONE, N_("Min Elev:"), VIK_LAYER_WIDGET_SPINBUTTON, param_scales + 0 },
-  { "max_elev", VIK_LAYER_PARAM_DOUBLE, VIK_LAYER_GROUP_NONE, N_("Max Elev:"), VIK_LAYER_WIDGET_SPINBUTTON, param_scales + 0 },
+  { "color", VIK_LAYER_PARAM_COLOR, VIK_LAYER_GROUP_NONE, N_("Min Elev Color:"), VIK_LAYER_WIDGET_COLOR, NULL, NULL, NULL  },
+  { "type", VIK_LAYER_PARAM_UINT, VIK_LAYER_GROUP_NONE, N_("Type:"), VIK_LAYER_WIDGET_RADIOGROUP_STATIC, params_type, NULL, NULL },
+  { "min_elev", VIK_LAYER_PARAM_DOUBLE, VIK_LAYER_GROUP_NONE, N_("Min Elev:"), VIK_LAYER_WIDGET_SPINBUTTON, &param_scales[0], NULL, NULL },
+  { "max_elev", VIK_LAYER_PARAM_DOUBLE, VIK_LAYER_GROUP_NONE, N_("Max Elev:"), VIK_LAYER_WIDGET_SPINBUTTON, &param_scales[0], NULL, NULL },
 };
 
 
@@ -131,9 +131,12 @@ static gboolean dem_layer_download_release ( VikDEMLayer *vdl, GdkEventButton *e
 static gboolean dem_layer_download_click ( VikDEMLayer *vdl, GdkEventButton *event, VikViewport *vvp );
 
 static VikToolInterface dem_tools[] = {
-  { N_("DEM Download/Import"), (VikToolConstructorFunc) dem_layer_download_create, NULL, NULL, NULL,
+  { { "DEMDownload", "vik-icon-DEM Download", N_("_DEM Download"), NULL, N_("DEM Download"), 0 },
+    (VikToolConstructorFunc) dem_layer_download_create, NULL, NULL, NULL,
     (VikToolMouseFunc) dem_layer_download_click, NULL,  (VikToolMouseFunc) dem_layer_download_release,
-    (VikToolKeyFunc) NULL, GDK_CURSOR_IS_PIXMAP, &cursor_demdl_pixbuf },
+    (VikToolKeyFunc) NULL,
+    FALSE,
+    GDK_CURSOR_IS_PIXMAP, &cursor_demdl_pixbuf },
 };
 
 
@@ -182,6 +185,8 @@ static const guint DEM_N_GRADIENT_COLORS = sizeof(dem_gradient_colors)/sizeof(de
 
 VikLayerInterface vik_dem_layer_interface = {
   "DEM",
+  N_("DEM"),
+  "<control><shift>D",
   &vikdemlayer_pixbuf,
 
   dem_tools,
@@ -323,7 +328,7 @@ static int dem_layer_load_list_thread ( dem_load_thread_data *dltd, gpointer thr
   //gdk_threads_enter();
   // Test is helpful to prevent Gtk-CRITICAL warnings if the program is exitted whilst loading
   if ( IS_VIK_LAYER(dltd->vdl) )
-    vik_layer_emit_update ( VIK_LAYER(dltd->vdl), TRUE ); // Yes update from background thread
+    vik_layer_emit_update ( VIK_LAYER(dltd->vdl) ); // NB update from background thread
   //gdk_threads_leave();
 
   return result;
@@ -371,18 +376,21 @@ gboolean dem_layer_set_param ( VikDEMLayer *vdl, guint16 id, VikLayerParamData d
       a_dems_list_free ( vdl->files );
       // Set file list so any other intermediate screen drawing updates will show currently loaded DEMs by the working thread
       vdl->files = data.sl;
-      // Thread Load
-      dem_load_thread_data *dltd = g_malloc ( sizeof(dem_load_thread_data) );
-      dltd->vdl = vdl;
-      dltd->vdl->files = data.sl;
+      // No need for thread if no files
+      if ( vdl->files ) {
+        // Thread Load
+        dem_load_thread_data *dltd = g_malloc ( sizeof(dem_load_thread_data) );
+        dltd->vdl = vdl;
+        dltd->vdl->files = data.sl;
 
-      a_background_thread ( VIK_GTK_WINDOW_FROM_WIDGET(vp),
-			    _("DEM Loading"),
-			    (vik_thr_func) dem_layer_load_list_thread,
-			    dltd,
-			    (vik_thr_free_func) dem_layer_thread_data_free,
-			    (vik_thr_free_func) dem_layer_thread_cancel,
-			    g_list_length ( data.sl ) ); // Number of DEM files
+        a_background_thread ( VIK_GTK_WINDOW_FROM_WIDGET(vp),
+                              _("DEM Loading"),
+                              (vik_thr_func) dem_layer_load_list_thread,
+                              dltd,
+                              (vik_thr_free_func) dem_layer_thread_data_free,
+                              (vik_thr_free_func) dem_layer_thread_cancel,
+                              g_list_length ( data.sl ) ); // Number of DEM files
+      }
       break;
     }
   }
@@ -427,7 +435,7 @@ VikDEMLayer *vik_dem_layer_new ( )
 {
   VikDEMLayer *vdl = VIK_DEM_LAYER ( g_object_new ( VIK_DEM_LAYER_TYPE, NULL ) );
 
-  vik_layer_init ( VIK_LAYER(vdl), VIK_LAYER_DEM );
+  vik_layer_set_type ( VIK_LAYER(vdl), VIK_LAYER_DEM );
 
   vdl->files = NULL;
 
@@ -566,7 +574,7 @@ static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *
     for ( x=start_x, counter.lon = start_lon; counter.lon <= end_lon+escale_deg*skip_factor; counter.lon += escale_deg * skip_factor, x += skip_factor ) {
       // NOTE: ( counter.lon <= end_lon + ESCALE_DEG*SKIP_FACTOR ) is neccessary so in high zoom modes,
       // the leftmost column does also get drawn, if the center point is out of viewport.
-      if ( x >= 0 && x < dem->n_columns ) {
+      if ( x < dem->n_columns ) {
         column = g_ptr_array_index ( dem->columns, x );
         // get previous and next column. catch out-of-bound.
 	gint32 new_x = x;
@@ -1092,7 +1100,7 @@ static void dem_download_thread ( DEMDownloadParams *p, gpointer threaddata )
     g_object_weak_unref ( G_OBJECT(p->vdl), weak_ref_cb, p );
 
     if ( dem_layer_add_file ( p->vdl, p->dest ) )
-      vik_layer_emit_update ( VIK_LAYER(p->vdl), TRUE ); // Yes update from background thread
+      vik_layer_emit_update ( VIK_LAYER(p->vdl) ); // NB update from background thread
   }
   g_mutex_unlock ( p->mutex );
   //gdk_threads_leave();
@@ -1158,7 +1166,7 @@ static gboolean dem_layer_download_release ( VikDEMLayer *vdl, GdkEventButton *e
     g_free ( tmp );
   }
   else
-    vik_layer_emit_update ( VIK_LAYER(vdl), FALSE );
+    vik_layer_emit_update ( VIK_LAYER(vdl) );
 
   g_free ( dem_file );
   g_free ( full_path );
