@@ -27,8 +27,12 @@
 
 #include "vikexttool.h"
 
+G_BEGIN_DECLS
+
 void vik_ext_tools_register ( VikExtTool *tool );
 void vik_ext_tools_unregister_all ();
 void vik_ext_tools_add_menu_items ( VikWindow *vwindow, GtkUIManager *uim );
 
+G_END_DECLS
+
 #endif
