@@ -32,9 +32,8 @@
 
 #include "viking.h"
 #include "util.h"
-#include "curl_download.h"
-
 #include "vikgototool.h"
+#include "vikgoto.h"
 
 static gchar *last_goto_str = NULL;
 static VikCoord *last_coord = NULL;
@@ -46,9 +45,8 @@ int last_goto_tool = 0;
 
 void vik_goto_register ( VikGotoTool *tool )
 {
-  IS_VIK_GOTO_TOOL( tool );
-
-  goto_tools_list = g_list_append ( goto_tools_list, g_object_ref ( tool ) );
+  if ( IS_VIK_GOTO_TOOL( tool ) )
+    goto_tools_list = g_list_append ( goto_tools_list, g_object_ref ( tool ) );
 }
 
 void vik_goto_unregister_all ()
@@ -155,7 +153,7 @@ static gchar *  a_prompt_for_goto_string(VikWindow *vw)
   return(goto_str);   /* goto_str needs to be freed by caller */
 }
 
-void a_vik_goto(VikWindow *vw, VikLayersPanel *vlp, VikViewport *vvp)
+void a_vik_goto(VikWindow *vw, VikViewport *vvp)
 {
   VikCoord new_center;
   gchar *s_str;
@@ -183,7 +181,6 @@ void a_vik_goto(VikWindow *vw, VikLayersPanel *vlp, VikViewport *vvp)
         g_free(last_successful_goto_str);
       last_successful_goto_str = g_strdup(last_goto_str);
       vik_viewport_set_center_coord(vvp, &new_center);
-      vik_layers_panel_emit_update(vlp);
       more = FALSE;
     }
     else if (!prompt_try_again(vw))
