@@ -24,13 +24,16 @@
 
 #include "vikwindow.h"
 #include "vikviewport.h"
-#include "viklayerspanel.h"
 #include "vikgototool.h"
 
+G_BEGIN_DECLS
+
 void vik_goto_register (VikGotoTool *tool);
 void vik_goto_unregister_all (void);
 
-extern void a_vik_goto(VikWindow *vw, VikLayersPanel *vlp, VikViewport *vvp);
+void a_vik_goto(VikWindow *vw, VikViewport *vvp);
 gchar * a_vik_goto_get_search_string_for_this_place(VikWindow *vw);
 
+G_END_DECLS
+
 #endif
