@@ -29,6 +29,8 @@
 #include "mapcoord.h"
 #include "vikmapslayer_compat.h"
 
+G_BEGIN_DECLS
+
 #define VIK_MAPS_LAYER_TYPE            (vik_maps_layer_get_type ())
 #define VIK_MAPS_LAYER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), VIK_MAPS_LAYER_TYPE, VikMapsLayer))
 #define VIK_MAPS_LAYER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), VIK_MAPS_LAYER_TYPE, VikMapsLayerClass))
@@ -47,10 +49,12 @@ typedef struct _VikMapsLayer VikMapsLayer;
 
 void maps_layer_init ();
 void maps_layer_register_map_source ( VikMapSource *map );
-void maps_layer_download_section_without_redraw ( VikMapsLayer *vml, VikViewport *vvp, VikCoord *ul, VikCoord *br, gdouble zoom);
+void maps_layer_download_section ( VikMapsLayer *vml, VikViewport *vvp, VikCoord *ul, VikCoord *br, gdouble zoom);
 gint vik_maps_layer_get_map_type(VikMapsLayer *vml);
 gchar *vik_maps_layer_get_map_label(VikMapsLayer *vml);
 gchar *maps_layer_default_dir ();
+void vik_maps_layer_download ( VikMapsLayer *vml, VikViewport *vvp, gboolean only_new );
 
+G_END_DECLS
 
 #endif
