@@ -49,11 +49,12 @@ struct _VikMapSourceClass
 	const gchar *(* get_license_url) (VikMapSource * self);
 	const GdkPixbuf *(* get_logo) (VikMapSource * self);
 
-	guint8 (* get_uniq_id) (VikMapSource * self);
+	guint16 (* get_uniq_id) (VikMapSource * self);
 	const gchar * (* get_label) (VikMapSource * self);
 	guint16 (* get_tilesize_x) (VikMapSource * self);
 	guint16 (* get_tilesize_y) (VikMapSource * self);
 	VikViewportDrawMode (* get_drawmode) (VikMapSource * self);
+	gboolean (* is_direct_file_access) (VikMapSource * self);
 	gboolean (* supports_download_only_new) (VikMapSource * self);
 	gboolean (* coord_to_mapcoord) (VikMapSource * self, const VikCoord * src, gdouble xzoom, gdouble yzoom, MapCoord * dest);
 	void (* mapcoord_to_center_coord) (VikMapSource * self, MapCoord * src, VikCoord * dest);
@@ -74,11 +75,12 @@ const gchar *vik_map_source_get_license (VikMapSource * self);
 const gchar *vik_map_source_get_license_url (VikMapSource * self);
 const GdkPixbuf *vik_map_source_get_logo (VikMapSource * self);
 
-guint8 vik_map_source_get_uniq_id (VikMapSource * self);
+guint16 vik_map_source_get_uniq_id (VikMapSource * self);
 const gchar *vik_map_source_get_label (VikMapSource * self);
 guint16 vik_map_source_get_tilesize_x (VikMapSource * self);
 guint16 vik_map_source_get_tilesize_y (VikMapSource * self);
 VikViewportDrawMode vik_map_source_get_drawmode (VikMapSource * self);
+gboolean vik_map_source_is_direct_file_access (VikMapSource * self);
 gboolean vik_map_source_supports_download_only_new (VikMapSource * self);
 gboolean vik_map_source_coord_to_mapcoord (VikMapSource * self, const VikCoord *src, gdouble xzoom, gdouble yzoom, MapCoord *dest );
 void vik_map_source_mapcoord_to_center_coord (VikMapSource * self, MapCoord *src, VikCoord *dest);
