@@ -26,7 +26,11 @@
 #include <gtk/gtk.h>
 #include "viktrwlayer.h"
 
+G_BEGIN_DECLS
+
 // To be only called from within viktrwlayer
 void trw_layer_geotag_dialog ( GtkWindow *parent, VikTrwLayer *vtl, VikTrack *track, const gchar *track_name );
 
+G_END_DECLS
+
 #endif
